#-------------------------------------------------------------.
# モーションのための便利機能.
# 要 : mathUtil.py
#-------------------------------------------------------------.

# ------------------------------------------------------------------.
# ボーンまたはボールジョイントのワールド座標での変換行列を計算.
# @param[in] bone_shape  ボーン形状の参照
# ------------------------------------------------------------------.
def getBoneWorldMatrix (bone_shape):
	lwMat         = bone_shape.local_to_world_matrix
	position      = bone_shape.bone_joint.position
	jointOffset   = bone_shape.bone_joint.offset
	jointRotation = bone_shape.bone_joint.rotation

	rotM = quaternion_matrix4x4(jointRotation)

	v  = add_vec3_vec3(position, jointOffset)
	m  = multiply_matrix4x4_matrix4x4(rotM, translate_matrix4x4(v))
	m  = multiply_matrix4x4_matrix4x4(m, lwMat)

	return m

# ------------------------------------------------------------------.
# ボーンまたはボールジョイントのワールド座標での中心位置を取得.
# @param[in] bone_shape  ボーン形状の参照
# ------------------------------------------------------------------.
def getBoneWorldCenterPosition (bone_shape):
	m = getBoneWorldMatrix(bone_shape)
	retV = multiply_vec_matrix4x4([0, 0, 0, 1], m)
	return retV

#-------------------------------------------------.
# 指定のジョイントの指定範囲のキーフレームを削除.
#-------------------------------------------------.
def deleteKeyframes (shape, start_frame, end_frame, depthF):
	if shape.has_motion == False:
		return

	motionPoints_count = shape.motion.get_number_of_motion_points()
	if motionPoints_count == 0:
		return

	for i in reversed(range(motionPoints_count)):
		mp = shape.motion.get_motion_point(i)
		if mp.sequence >= start_frame and mp.sequence <= end_frame:
			mp.delete()

	if depthF:
		if shape.has_son:
			s = shape.son
			while s.has_bro:
				s = s.bro
				deleteKeyframes(s, start_frame, end_frame, depthF)

