#-------------------------------------------------------------.
# モーション情報を更新.
# @param[in]  start_frame     開始フレーム.
# @param[in]  end_frame       終了フレーム.
# @param[in]  frame_interval  キーフレーム間隔.
# @param[in]  target_bone     ターゲットボーン名.
# @param[in]  src_bone        元のボーン名.
# @param[in]  rotation_v      回転の影響度（0.0 - 1.0）。0.0で回転補正がかからない.
#-------------------------------------------------------------.

# ------------------------------------------------------------------.
# 対象となる2つのボーンを取得.
targetShape = xshade.scene().get_shape_by_name(target_bone, True)
srcShape    = xshade.scene().get_shape_by_name(src_bone, True)

start_frame    = int(start_frame)
end_frame      = int(end_frame)
frame_interval = int(frame_interval)

firstDir    = [0.0, 0.0, 0.0]
firstRotM   = identity_matrix4x4()

old_sequence = xshade.scene().sequence_value

targetShape.motion.prohibit_update()	# モーションウィンドウの更新禁止.

# start_frame から end_frame にかけて、
# targetShapeのボーンをsrcWPosの位置に追従するようにオフセット移動.

scale     = identity_vec3()
shear     = identity_vec3()
rotate    = identity_vec3()
translate = identity_vec3()

tRotM = identity_matrix4x4()

interval = 0
for sequencePos in range(start_frame, end_frame + 1):
	if (interval % frame_interval) != 0 and sequencePos != end_frame:
		interval += 1
		continue

	# シーケンス位置を移動
	xshade.scene().sequence_value = sequencePos

	# ボーン中心のワールド座標位置を計算.
	srcWMatrix = getBoneWorldMatrix(srcShape)
	srcWPos    = multiply_vec_matrix4x4([0, 0, 0, 1], srcWMatrix)
	targetWPos = getBoneWorldCenterPosition(targetShape)
	dd         = sub_vec3_vec3(srcWPos, targetWPos)		# 移動量

	# targetShapeでのローカル座標値に変換 (回転値の格納時に使用).
	tLWMatrix  = set_matrix4x4(targetShape.local_to_world_matrix)
	localMatrix = multiply_matrix4x4_matrix4x4(inverse_matrix4x4(tLWMatrix), srcWMatrix)
	
	# srcShapeのワールド座標での回転要素を取得.
	unmatrix_matrix4x4(localMatrix, scale, shear, rotate, translate)
	rotM = rotate_matrix4x4(rotate)

	if sequencePos == start_frame:
		# 回転要素.
		tRotM = getBoneWorldMatrix(targetShape)
		tRotM = multiply_matrix4x4_matrix4x4(inverse_matrix4x4(tLWMatrix), tRotM)
		unmatrix_matrix4x4(tRotM, scale, shear, rotate, translate)
		tRotM = rotate_matrix4x4(rotate)

		firstDir  = dd
		firstRotM = rotM

		# targetShapeでの指定範囲のキーフレームを削除.
		deleteKeyframes(targetShape, start_frame, end_frame, False)
	
	rotM = multiply_matrix4x4_matrix4x4(rotM, tRotM)

	# ワールド座標での回転の差分を計算.
	diffRotM = multiply_matrix4x4_matrix4x4(inverse_matrix4x4(firstRotM), rotM)

	unmatrix_matrix4x4(diffRotM, scale, shear, rotate, translate)
	rotate *= float(rotation_v)   	# rotation_vは、回転を反映させる影響度.
	diffRot = matrix4x4_quaternion(rotate_matrix4x4([ rotate[0], rotate[1], rotate[2] ]))

	# ジョイントとしてのオフセット値の更新.
	mIndex = targetShape.motion.create_motion_point(sequencePos)
	mp = targetShape.motion.get_motion_point(mIndex)
	offsetV = sub_vec3_vec3(dd, firstDir)
	offset = [0.0, 0.0, 0.0]
	offset[0] = mp.offset[0] + offsetV[0]
	offset[1] = mp.offset[1] + offsetV[1]
	offset[2] = mp.offset[2] + offsetV[2]
	mp.offset = offset

	# ジョイントとしての回転値の更新.
	mp.rotation = diffRot.tolist()

	interval += 1

xshade.scene().sequence_value = old_sequence

targetShape.motion.resume_update()	    # モーションウィンドウの更新再開.

result = ""
